#! /usr/bin/env python

# 
# Copyright © 2000-2001, GMD, Sankt Augustin
# -- German National Research Center for Information Technology 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

import string
from dvbobjects.utils import *

######################################################################
class SuperGroupSpec(DVBobject):

    def read(self, filename):
        self.PATH = filename
        file = open(filename)

        items = string.split(file.readline())
        self.transactionId = eval(items[0])
	self.version       = eval(items[1])
        fn = items[1]
        if fn == "None":
            self.srg_ior = None
        else:
            self.srg_ior       = open(items[2]).read()

        self.groups = []
        
        while (1):
            line = file.readline()
            if not line: break

            group = GroupSpec()
            #print repr(line)
            group.read(line)
            self.groups.append(group)

    def write(self, outputDir):
        specFile      = "%s/DSI.spec" % outputDir
        dsi = open(specFile, "wb")
        dsi.write("0x%08X 0x%08X %s\n" % (
                  int(self.transactionId),
		  self.version,
                  self.srg_ior,
                  ))
        for group in self.groups:
            group.write(dsi, outputDir)

    def addGroup(self, **kwargs):
        group = apply(GroupSpec, [], kwargs)
        try:
            self.groups.append(group)
        except AttributeError:
            self.groups = [group]

    def addModule(self, **kwargs):
        curr = self.groups[-1]
        apply(GroupSpec.addModule, (curr,), kwargs)

######################################################################
class GroupSpec(DVBobject):

    def read(self, specline):
        items = string.split(specline)

        self.PATH=items[0]
        #print repr(items[0])
        diiSpecFile        = open(items[0])
        self.transactionId = eval(items[1])
	self.version       = eval(items[2])
        self.downloadId    = eval(items[3])
        if len(items) == 6:
            self.assocTag      = eval(items[4])
            self.blockSize     = eval(items[5])
        else:
            self.assocTag      = None
            self.blockSize     = eval(items[6])

        self.modules = []

        while 1:
            line = diiSpecFile.readline()
            if not line:
                break

            mod = ModuleSpec()
            mod.read(line)
            self.modules.append(mod)

    def write(self, specFile, outputDir):
        specFile.write(
            "%s 0x%08X 0x%08X 0x%08X 0x%04X %4d\n" % (
            "%s/DII.spec" % outputDir,
            int(self.transactionId),
	    self.version,
            self.downloadId,
            self.assocTag,
            self.blockSize,
            ))

    def addModule(self, **kwargs):
        mod = apply(ModuleSpec, (), kwargs)
        try:
            self.modules.append(mod)
        except AttributeError:
            self.modules = [mod]

######################################################################
class ModuleSpec(DVBobject):

    def read(self, specline):
            items = string.split(specline)
            #print repr(items[0])
            #print repr(items[1])
            self.INPUT          = items[0]
            self.tableid        = eval(items[1])
            self.moduleId       = eval(items[2])
            self.moduleVersion  = eval(items[3])

######################################################################





#! /usr/bin/env python

# This file is part of the dvbobjects library.
# 
# Copyright  2009-2013, Lorenzo Pallara l.pallara@avalpa.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

import string

from dvbobjects.utils import *
from dvbobjects.utils.Hamming import *
from dvbobjects.utils.ByteParity import *
from dvbobjects.utils.ByteInvert import *

######################################################################
class EBUTeletext(DVBobject):
	"""The class for EBU Teletext data units, NB. they are not sections
	"""
	def pack(self):
		if (self.data_unit_id == 0xFF):
#			return pack("!BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB", 
			return pack("BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB", 
				    0xFF, 0x2C, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
				    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
				    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
				    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF,
				    0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF
				    )
		b3 = self.row & 0x1 
		b2 = (self.magazine >> 2) & 0x01  
		b1 = (self.magazine >> 1) & 0x01
		b0 = self.magazine & 0x1
		hamming1 = invert(hamming84((b3 << 3) | (b2 << 2) | (b1 << 1) | b0))
		b3 = (self.row >> 4) & 0x01
		b2 = (self.row >> 3) & 0x01
		b1 = (self.row >> 2) & 0x01
		b0 = (self.row >> 1) & 0x01
		hamming2 = invert(hamming84((b3 << 3) | (b2 << 2) | (b1 << 1) | b0))
		payload = ""
		if (self.row == 0) :
			for i in range(0, len(self.chars)):
				#temp1 = ord(self.chars[i])
				#temp2 = oddparity(temp1)
				#temp3 = invert(temp2)
				#temp4 = temp3 & 0xFF
				#payload += pack("B", temp4)
				#payload += pack("!B", invert(oddparity(ord(self.chars[i]))))
				payload += pack("B", invert(oddparity(ord(self.chars[i]))))
			assert len(payload) == 32
			hamming3 = invert(hamming84(self.page & 0xF))
			hamming4 = invert(hamming84((self.page >> 4) & 0xF))
			hamming5 = invert(hamming84(self.subpage & 0xF))
			b3 = self.erase_page & 0x1
			b2 = (self.subpage >> 6 ) & 0x1
			b1 = (self.subpage >> 5 ) & 0x1
			b0 = (self.subpage >> 4 ) & 0x1
			hamming6 = invert(hamming84((b3 << 3) | (b2 << 2) | (b1 << 1) | b0))
			b3 = (self.subpage >> 10 ) & 0x1
			b2 = (self.subpage >> 9 ) & 0x1
			b1 = (self.subpage >> 8 ) & 0x1
			b0 = (self.subpage >> 7 ) & 0x1
			hamming7 = invert(hamming84((b3 << 3) | (b2 << 2) | (b1 << 1) | b0))
			b3 = self.subtitle & 0x1
			b2 = self.newsflash & 0x1
			b1 = (self.subpage >> 12 ) & 0x1
			b0 = (self.subpage >> 11 ) & 0x1
			hamming8 = invert(hamming84((b3 << 3) | (b2 << 2) | (b1 << 1) | b0))
			b3 = self.inhibit_display & 0x1
			b2 = self.interrupted_sequence & 0x1
			b1 = self.update_indicator & 0x1
			b0 = self.suppress_header & 0x1
			hamming9 = invert(hamming84((b3 << 3) | (b2 << 2) | (b1 << 1) | b0))
			b3 = self.country_code & 0x1
			b2 = (self.country_code >> 1) & 0x1
			b1 = (self.country_code >> 2) & 0x1
			b0 = self.magazine_serial & 0x1
			hamming10 = invert(hamming84((b3 << 3) | (b2 << 2) | (b1 << 1) | b0))
#			return pack("!BBBBBBBBBBBBBB",
			return pack("BBBBBBBBBBBBBB",
				self.data_unit_id, #0x02 non-subtile or 0x03 for subtitles
				0x2C, # 44 bytes
				0x01 << 6 | (self.field_parity & 0x01) << 5 | self.line_offset & 0x1F,
				0xE4, # frame coding EBU Teletext
				hamming1,
				hamming2,
				hamming3,
				hamming4,
				hamming5,
				hamming6,
				hamming7,
				hamming8,
				hamming9,
				hamming10
			) + payload
		elif (self.row == 27) :
			link1bytes = self.link1.pack()
			link2bytes = self.link2.pack()
			link3bytes = self.link3.pack()
			link4bytes = self.link4.pack()
			link5bytes = self.link5.pack()
			link6bytes = self.link6.pack()
			crc = 0x0000 # not used 
#			fmt = "!BBBBBBB%ds%ds%ds%ds%ds%dsBH" % (len(link1bytes), len(link2bytes), len(link3bytes), len(link4bytes), len(link5bytes), len(link6bytes))
			fmt = "BBBBBBB%ds%ds%ds%ds%ds%dsBH" % (len(link1bytes), len(link2bytes), len(link3bytes), len(link4bytes), len(link5bytes), len(link6bytes))
			return pack(fmt,
				self.data_unit_id, #0x02 non-subtile or 0x03 for subtitles
				0x2C, # 44 bytes
				0x01 << 6 | (self.field_parity & 0x01) << 5 | self.line_offset & 0x1F,
				0xE4, # frame coding EBU Teletext
				hamming1,
				hamming2,
				invert(hamming84(self.designation_code)), # designation code allows 2^4 = 16 different pages more
				link1bytes,
				link2bytes,
				link3bytes,
				link4bytes,
				link5bytes,
				link6bytes,
				invert(hamming84(0xE)), # LC
				crc
			)
		else :
			for i in range(0, len(self.chars)):
#				payload += pack("!B", invert(oddparity(ord(self.chars[i]))))
				payload += pack("B", invert(oddparity(ord(self.chars[i]))))
			assert len(payload) == 40
#			return pack("!BBBBBB",
			return pack("BBBBBB",
				self.data_unit_id, #0x02 non-subtile or 0x03 for subtitles
				0x2C, # 44 bytes
				0x01 << 6 | (self.field_parity & 0x01) << 5 | self.line_offset & 0x1F,
				0xE4, # frame coding EBU Teletext
				hamming1,
				hamming2,
			) + payload # 40 characters of a line, a page is 24 lines
		
		
######################################################################
class EBUTeletextUnits(DVBobject):
	def pack(self):
	
		# pack unit_loop
		pl_bytes = string.join(
			map(lambda x: x.pack(),
			self.unit_loop),
			"")
		fmt = "!%ds" % (len(pl_bytes))
		return pack(fmt, 
			pl_bytes
		)

class EBUTeletextLink(DVBobject):
	def pack(self):
		hamming1 = invert(hamming84(self.page & 0xF))
		hamming2 = invert(hamming84((self.page >> 4) & 0xF))
		hamming3 = invert(hamming84(self.subpage & 0xF))
		b3 = not (self.magazine & 0x1)
		b2 = (self.subpage >> 6 ) & 0x1
		b1 = (self.subpage >> 5 ) & 0x1
		b0 = (self.subpage >> 4 ) & 0x1
		hamming4 = invert(hamming84((b3 << 3) | (b2 << 2) | (b1 << 1) | b0))
		b3 = (self.subpage >> 10 ) & 0x1
		b2 = (self.subpage >> 9 ) & 0x1
		b1 = (self.subpage >> 8 ) & 0x1
		b0 = (self.subpage >> 7 ) & 0x1
		hamming5 = invert(hamming84((b3 << 3) | (b2 << 2) | (b1 << 1) | b0))
		b3 = not ((self.magazine >> 1) & 0x1)
		b2 = not ((self.magazine >> 2) & 0x1)
		b1 = (self.subpage >> 12 ) & 0x1
		b0 = (self.subpage >> 11 ) & 0x1
		hamming6 = invert(hamming84((b3 << 3) | (b2 << 2) | (b1 << 1) | b0))
#		fmt = "!BBBBBB" 
		fmt = "BBBBBB" 
		return pack(fmt, 
			hamming1,
			hamming2,
			hamming3,
			hamming4,
			hamming5,
			hamming6,
		)




#! /usr/bin/env python

# 
# Copyright  2000-2001, GMD, Sankt Augustin
# -- German National Research Center for Information Technology 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import sys
import os
import re
import getopt
import pprint
import string

from stat import ST_SIZE
from dvbobjects.utils import *
from dvbobjects.utils.SpecFile import *
from dvbobjects.DSMCC import BIOP
from dvbobjects.DVB.DataCarousel import TransactionId

######################################################################

TIMEOUT			= 0xFFFFFFFF
TABLE_ID		= 0x3C
MAX_MODULE_SIZE		= 1024 * 64

DEBUG 			= 0
OPTIONS = "h"
LONG_OPTIONS = [ "help" ]

######################################################################
class ModuleBuilder:

	def __init__(self, OUTPUT_DIR, MODULE_ID):
		self.module_id = MODULE_ID
		filename = "%s/%04d.mod" % (OUTPUT_DIR, self.module_id)
		self.name = filename # needed for debug
		if DEBUG:
			print "NEW MODULE: %s" % filename
		self.__file = open(filename, "wb")
		self.size = 0

	def hasRoom(self, requestedSize):
		if self.size == 0:
			# Anything goes...
			return 1
		elif self.size + requestedSize <= MAX_MODULE_SIZE:
			# Module isn't empty, but still have space
			return 1
		else:
			# Start new module
			return 0

	def write(self, bytes):
		msgSize = len(bytes)
		if DEBUG:
			print "ADD (mod %d, size %d+%d=%d)" % ( self.module_id, self.size, msgSize, self.size+msgSize)
		self.__file.write(bytes)
		self.size = self.size + msgSize

######################################################################
class ObjectCarouselBuilder: # Build an object carousel with 1 DII, this limits the number of modules to something a bit more than 100

	def __init__(self, OUTPUT_DIR, CAROUSEL_ID, DOWNLOAD_ID, ASSOC_TAG, MODULE_VERSION, BLOCK_SIZE, UPDATE_FLAG, COMPRESS_MODE):
		
		self.MODULE_ID = 2
		
		# The builder generate modules files and specification files
		# specification files are for informations not included in the modules files themself
		
		self.DSI_TransactionId = TransactionId(
			version = MODULE_VERSION,
			identification = 0,
			updateFlag     = UPDATE_FLAG,
			)
		
		self.DII_TransactionId = TransactionId(
			version = MODULE_VERSION,
			identification = 1,
			updateFlag     = UPDATE_FLAG,
			)
		
		self.__SPECdii = open("%s/DII.spec" % OUTPUT_DIR, "w")
		
		self.__spec = SuperGroupSpec(
			transactionId = self.DSI_TransactionId,
			version       = MODULE_VERSION,
			srg_ior       = "%s/SRG_IOR" % OUTPUT_DIR,
		)

		self.__spec.addGroup(
			transactionId = self.DII_TransactionId,
			version       = MODULE_VERSION,
			downloadId    = DOWNLOAD_ID,
			assocTag      = ASSOC_TAG,
			blockSize     = BLOCK_SIZE,
			)
	
		# Output variables
		self.OUTPUT_DIR = OUTPUT_DIR
		self.DOWNLOAD_ID = DOWNLOAD_ID
		self.CAROUSEL_ID = CAROUSEL_ID
		self.ASSOC_TAG = ASSOC_TAG
		self.MODULE_VERSION = MODULE_VERSION
		self.COMPRESS_MODE = COMPRESS_MODE
		self.UPDATE_FLAG = UPDATE_FLAG
		self.BLOCK_SIZE = BLOCK_SIZE
		
		# currently open modules indexed by type
		self.__ModByType = {
			'sgw': 0,
			'dir': 0,
			'fil': 0,
			'ste': 0,
			'solo': 0,
		}
		self.__TypeInUse = {
			'sgw': 0,
			'dir': 0,
			'fil': 0,
			'ste': 0,
			'solo': 0,
		}
		
		self.__ModById = {}

		# Table of Contents, for debugging
		self.TOC = []

	def genSpec(self):
		self.__spec.write(self.OUTPUT_DIR)
	
	def addDirectory(self, node):
		self.__addNode(node, "dir")
	
	def addSGW(self, node):
		self.__addNode(node, "sgw")
		
	def addFile(self, node):
		self.__addNode(node, "fil")
		
	def addSoloFile(self, node):
		self.__addNode(node, "solo")
		
	def addStreamEvent(self, node):
		self.__addNode(node, "ste")
	
	def __addNode(self, node, type):
		msg = node.message()
		msgBytes = msg.pack()
		msgSize = len(msgBytes)
		
		if (type == 'solo'):
			self.__ModByType[type] = self.__nextModule(self.MODULE_ID)
			self.__ModByType[type].write(msgBytes)
			filename = "%s/%04d.mod.solo" % (OUTPUT_DIR, self.__ModByType[type].module_id)
			out = open(filename, "wb")
			out.close()
			modid = str(self.__ModByType[type].module_id)
		elif (type == 'sgw'):
			self.__ModByType[type] = self.__nextModule(1)
			self.__ModByType[type].write(msgBytes)
			modid = str(self.__ModByType[type].module_id)
		else:
			if not self.__TypeInUse[type]:
				self.__ModByType[type] = self.__nextModule(self.MODULE_ID)
				self.__TypeInUse[type] = 1
		
			if self.__ModByType[type].hasRoom(msgSize):
				self.__ModByType[type].write(msgBytes)
			else:
				self.__ModByType[type] = self.__nextModule(self.MODULE_ID)
				self.__ModByType[type].write(msgBytes)
			modid = str(self.__ModByType[type].module_id)
			
		node.bind(
			carouselId = self.CAROUSEL_ID,
			moduleId   = int(modid),
			assoc_tag = self.ASSOC_TAG,
			DII_TransactionId = self.DII_TransactionId,
		)
	
		self.TOC.append((
			int(modid),
			msg.objectKey,
			os.path.basename(msg.PATH),
			msg.objectKind,
			msgSize,
		))

	def __nextModule(self, modid):
		mod = ModuleBuilder(self.OUTPUT_DIR, int(modid))
		if (int(modid) > 1): # id 1 is reserved for sgw
			self.MODULE_ID = mod.module_id + 1
		
		self.__SPECdii.write("%s 0x%02X 0x%04X 0x%02X\n" % (
			mod.name,
			TABLE_ID,
			mod.module_id, 
			self.MODULE_VERSION))
		
		self.__spec.addModule(
			tableId       = TABLE_ID,
			moduleId      = mod.module_id,
			moduleVersion = self.MODULE_VERSION,
		)
		
		return mod

######################################################################
class FSNode(DVBobject): # superclass for FSDirectory, FSSteam and FSFile.

	def __init__(self, KEY_SERIAL_NUMBER):
		self.KEY = KEY_SERIAL_NUMBER
	def IOR(self, carouselId, moduleId, key, assoc_tag, DII_TransactionId):
		iop = BIOP.IOP.IOR(
			PATH = self.PATH, # for debugging
			type_id = self.MessageClass.objectKind,
			carouselId = carouselId,
			moduleId   = moduleId,
			objectKey  = key,
			assocTag      = assoc_tag,
			transactionId = DII_TransactionId,
			timeout       = TIMEOUT,
		)
		return iop

	def _checkBinding(self):
		try:
			raise "Already Bound", self._binding
		except AttributeError:
			pass
			

######################################################################
class FSFile(FSNode): # A File in a File System destined for an Object Carousel.

	MessageClass = BIOP.FileMessage
	BindingClass = BIOP.ObjectFileBinding
	
	def __init__(self, path, KEY_SERIAL_NUMBER):
		FSNode.__init__(self, KEY_SERIAL_NUMBER)
		assert(len(path) > 0)
		self.PATH = path
		self.contentSize = os.stat(path)[ST_SIZE]
	
	def bind(self, carouselId, moduleId, assoc_tag, DII_TransactionId):
		self._checkBinding()
		filename = os.path.basename(self.PATH)
		self._binding = self.BindingClass(
			nameId = filename + "\x00",
			IOR = self.IOR(carouselId, moduleId, self.KEY, assoc_tag, DII_TransactionId),
			contentSize = self.contentSize,
		)
	
	def binding(self):
		return self._binding
	
	def message(self):
		msg = self.MessageClass(
			PATH = self.PATH,
			objectKey   = self.KEY,
			contentSize = self.contentSize,
		)
		return msg
	
	def shipMessage(self, theObjectCarouselBuilder, isSolo):
		if (isSolo == 1):
			theObjectCarouselBuilder.addSoloFile(self)
		else:
			theObjectCarouselBuilder.addFile(self)
		
######################################################################
class FSStreamEvent(FSNode): # A Directory in a File System destined to genereate a StreamEvent Object for Object Carousel.

	MessageClass = BIOP.StreamEventMessage
	BindingClass = BIOP.ObjectStreamEventBinding
	
	def __init__(self, path, KEY_SERIAL_NUMBER):
		FSNode.__init__(self, KEY_SERIAL_NUMBER)
		assert(len(path) > 0)
		self.PATH = path
		
	def bind(self, carouselId, moduleId, assoc_tag, DII_TransactionId):
		self._checkBinding()
		filename = os.path.basename(self.PATH)
		self._binding = self.BindingClass(
			nameId = filename + "\x00",
			IOR = self.IOR(carouselId, moduleId, self.KEY, assoc_tag, DII_TransactionId),
		)
	
	def binding(self):
		return self._binding
	
	def message(self):
		msg = self.MessageClass(
			PATH = self.PATH,
			objectKey   = self.KEY,
		)
		return msg
	
	def shipMessage(self, theObjectCarouselBuilder):
		theObjectCarouselBuilder.addStreamEvent(self)

######################################################################
class FSDir(FSNode, ObjectCarouselBuilder):# A Directory in a File System destined for an Object Carousel.

	MessageClass = BIOP.DirectoryMessage
	BindingClass = BIOP.ContextBinding
	
	def __init__(self, path, KEY_SERIAL_NUMBER):
		FSNode.__init__(self, KEY_SERIAL_NUMBER)
		assert(len(path) > 0)
		self.PATH  = path
		self.bindings = []
		self.visitKEY = KEY_SERIAL_NUMBER
	
	def bind(self, carouselId, moduleId, assoc_tag, DII_TransactionId):
		self._checkBinding()
		filename = os.path.basename(self.PATH)
		self._binding = self.BindingClass(
			nameId = filename + "\x00",
			IOR = self.IOR(carouselId, moduleId, self.KEY, assoc_tag, DII_TransactionId),
		)
	
	def binding(self):
		return self._binding
		
	
	def message(self):
		msg = self.MessageClass(
			PATH = self.PATH,
			objectKey = self.KEY,
			bindings = self.bindings,
		)
		return msg 
	
	
	def visit(self, theObjectCarouselBuilder): #Depth first visit

		#REJECT_EXT = ['.pyc', '.o', '.so']
		#SINGLE_MODULE_EXT = ['.jpg', '.png', '.gif', '.solo']
		SINGLE_MODULE_EXT = ['.solo']
		#REJECT_FN = ['x', 'tmp']
		#REJECT_FN = []
		EVENT_EXT = ['.event']
	
		assert os.path.isdir(self.PATH), self.PATH
		try:
			ls = os.listdir(self.PATH)
		except:
			print self.PATH
			raise
	
		ls.sort()
	
		for filename in ls:
			path = os.path.join(self.PATH, filename)
#			if os.path.splitext(filename)[1] in REJECT_EXT:
#				continue
			if os.path.isfile(path):
				self.visitKEY = self.visitKEY + 1
				obj = FSFile(path, self.visitKEY)
				if (os.path.splitext(filename)[1] in SINGLE_MODULE_EXT) and (theObjectCarouselBuilder.COMPRESS_MODE == 2):
					obj.shipMessage(theObjectCarouselBuilder, 1)
				else:
					obj.shipMessage(theObjectCarouselBuilder, 0)
				if DEBUG:
					print obj.message()
					print
			elif os.path.isdir(path):
				if os.path.splitext(filename)[1] in EVENT_EXT:
					self.visitKEY = self.visitKEY + 1
					obj = FSStreamEvent(path, self.visitKEY)
					obj.shipMessage(theObjectCarouselBuilder)
					if DEBUG:
						print obj.message()
						print
				else:
					self.visitKEY = self.visitKEY + 1
					obj = FSDir(path, self.visitKEY)
					obj.visit(theObjectCarouselBuilder)
					self.visitKEY = obj.visitKEY
					if DEBUG:
						print obj.message()
						print
			else:
				continue
		
			self.bindings.append(obj.binding())
	
		# THIS directory (i.e. self) is complete, so...
		self.shipMessage(theObjectCarouselBuilder)
		if DEBUG:
			print self.message()
			print
		
	
	def shipMessage(self, theObjectCarouselBuilder):
		theObjectCarouselBuilder.addDirectory(self)

######################################################################
class FSRoot(FSDir): #A Directory in a File System destined as Service Gateway for an Object Carousel
	MessageClass = BIOP.ServiceGatewayMessage
	
	def shipMessage(self, theObjectCarouselBuilder):
		theObjectCarouselBuilder.addSGW(self)

######################################################################
if __name__ == '__main__':
	try:
		opts, args = getopt.getopt(sys.argv[1:], OPTIONS, LONG_OPTIONS)
	except getopt.error:
		print ("Usage: %s"
			" <InputDirectory>"
			" <OutputModulesDirectory>"
			" download_id"
			" carousel_id"
			" association_tag"
			" version" 
			" [block_size update_flag compress_mode]"
			) % (
		sys.argv[0])
		sys.exit(1)
	
	INPUT_DIR, OUTPUT_DIR, CAROUSEL_ID, DOWNLOAD_ID, ASSOC_TAG, MODULE_VERSION, BLOCK_SIZE, UPDATE_FLAG, COMPRESS_MODE = args
	
	root = FSRoot(INPUT_DIR, 0)
	theObjectCarouselBuilder = ObjectCarouselBuilder(OUTPUT_DIR, int(CAROUSEL_ID), int(DOWNLOAD_ID), int(ASSOC_TAG, 16), int(MODULE_VERSION, 16), int(BLOCK_SIZE), int(UPDATE_FLAG), int(COMPRESS_MODE))
	root.visit(theObjectCarouselBuilder)
	
	out = open("%s/SRG_IOR" % OUTPUT_DIR, "wb")
	out.write(root.binding().IOR.pack())
	out.close()
	
	theObjectCarouselBuilder.genSpec()
	
	if DEBUG:
		print root.binding().IOR
		pprint.pprint(theObjectCarouselBuilder.TOC)



#
# Copyright 2010, LIFIA - Facultad de Informatica - Univ. Nacional de La Plata
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#                                  
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#                                  
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

import string
from dvbobjects.utils.DVBobject import *
from dvbobjects.MPEG.Descriptor import *
from dvbobjects.MPEG.Section import Section
from dvbobjects.utils.MJD import *

# @author Ignacio Jaureguiberry <ijaureguiberry@lifia.info.unlp.edu.ar>

# TODO Lots of testing, most classes needs to be modified
#      so that they work with default values.
# TODO Clean up a bit.

#
# SDTT as defined in ARIB STD-B21 - ver 4.6-E1 - Table 12-1
#
class software_download_trigger_table(Section):
    
    table_id = 0xC3

    def pack_section_body(self):

        self.table_id_extension = (self.maker_id << 8) | (self.model_id & 0xff)

        content_bytes = string.join(
            map(lambda x: x.pack(),
                self.contents),
            ""
        )

        fmt = ("!"
            "H" # transport_stream_id: 16 bits
            "H" # original_network_id: 16
            "H" # service_id: 16
            "B" # number_of_contents: 8
            "%ds" # contents()
              % len(content_bytes))
        return pack (fmt,
            self.transport_stream_id,
            self.original_network_id,
            self.service_id,
            len(self.contents), # number of contents, should be length in bytes instead of qty.??
            content_bytes)

class sdtt_content_loop_item(DVBobject):
        
    def pack(self):
        schedule_loop_bytes = string.join(
            map(lambda x: x.pack(), self.schedule_loop_items), ""
        )
        descriptor_loop_bytes = string.join(
            map(lambda x: x.pack(), self.descriptors), ""
        )

        fmt = ("!"
            "H" # group: 4 bits, target_version: 12
            "H" # new_version: 12, download_level: 2, version_indicator: 2
            "H" # content_descriptor_length: 12, reserved: 4
            "H" # schedule_descriptor_length: 12, schedule_timeshift_information: 4
            "%ds" # loop: start_time: 40, duration: 24
            "%ds" # loop: descriptors
            % (len(schedule_loop_bytes),len(descriptor_loop_bytes)))
        return pack(fmt,
            (self.group       << 12) | (self.target_version),
            (self.new_version <<  4) | (self.download_level << 2) | self.version_indicator,
            ( (len(schedule_loop_bytes)+len(descriptor_loop_bytes)) << 4) | 0x0F,
            ( (len(schedule_loop_bytes)) << 4) | self.schedule_timeshift_information,
            schedule_loop_bytes,
            descriptor_loop_bytes)


class sdtt_schedule_loop_item(DVBobject):
    def pack(self):
        fmt = ("!HBBBBBB")
        return pack(fmt,
            int(MJD_convert(self.start_year, self.start_month, self.start_day)),
            self.start_hour,
            self.start_minute,
            self.start_second,
            self.duration_hours,
            self.duration_minutes,
            self.duration_seconds,
        )

class download_content_descriptor(Descriptor):

    descriptor_tag = 0xC9

    def has(self, bytes):
        if len(bytes) > 0:
            return 1
        return 0

    def bytes(self):
        # TODO clean up...
        if self.compatibility_descriptor_bytes == None:
            self.compatibility_descriptor_bytes = "\000\000"
        if self.modules_info_bytes == None:
            self.modules_info_bytes             = ""

        if len(self.text_data) > 0 and len(self.text_ISO639_lang) == 3:
            text_data_bytes = pack(
                "3sB%ds" % len(self.text_data),
                self.text_ISO639_lang,
                len(self.text_data), self.text_data
            )
        else:
            text_data_bytes = ""

        fmt = ("!"
            "B" # reboot:1, add_on:1, compatibility_flag=1, module_info_flag=1, text_info_flag=1, reserved=3
            "L" # component_size:32
            "L" # download_id: 32
            "L" # time_out_value_DII: 32
            "BH" # leak_rate: 22, reserved: 2
            "B" # component_tag: 8
            "H%ds" # compatibility_descriptor: if compatibility_flag=1
            "%ds" # modules_info: if module_info_flag=1
            "B" # private_data_length: 8
            "%ds" # private_data_bytes
            "%ds" # text_data_bytes: if text_info_flag=1
            % (len(self.compatibility_descriptor_bytes),
            len (self.modules_info_bytes),
            len (self.privateData),
            len (text_data_bytes)
            ))
        return pack(
            fmt,
                (self.reboot << 7) |
                (self.add_on << 6) |
                (self.has(self.compatibility_descriptor_bytes)) << 5 |
                (self.has(self.modules_info_bytes)) << 4 |
                (self.has(text_data_bytes)) << 3 |
                0x07, # 3x1 bit for reserved field.
                self.component_size, # sum of data sizes transmitted by the carousel in bytes.
                self.download_id,     # same as the one transmitted in DII/DDB
                self.time_out_value_DII, # timeout value in ms for all DII sections of the carousel.
                self.leak_rate >> 14, ((self.leak_rate << 2) | 0x3)&0xFFFF, # leak_rate. # TODO CHECK its ok...
                self.component_tag,
		len(self.compatibility_descriptor_bytes),
                self.compatibility_descriptor_bytes,
                self.modules_info_bytes,
                len(self.privateData),
                self.privateData,
                text_data_bytes
            )



#! /usr/bin/env python

# This file is part of the dvbobjects library.
# 
# Copyright © 2004-2013 Lorenzo Pallara l.pallara@avalpa.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

import string
from dvbobjects.MPEG.Section import Section
from dvbobjects.utils import *
from dvbobjects.DVB.Descriptors import *

######################################################################
class service_description_section(Section):

    table_id = 0x42
    
    section_max_size = 1024

    def pack_section_body(self):

        self.table_id_extension = self.transport_stream_id
        self.private_indicator = 1
    
        # pack service_stream_loop
        sl_bytes = string.join(
            map(lambda x: x.pack(),
                self.service_loop),
            "")

        fmt = "!HB%ds" % len(sl_bytes)
        return pack(fmt,
	    self.original_network_id,
	    0xFF,
            sl_bytes,
            )

######################################################################
class service_description_other_ts_section(Section):

    table_id = 0x46
    
    section_max_size = 1024

    def pack_section_body(self):

        self.table_id_extension = self.transport_stream_id
        self.private_indicator = 1
    
        # pack service_stream_loop
        sl_bytes = string.join(
            map(lambda x: x.pack(),
                self.service_loop),
            "")

        fmt = "!HB%ds" % len(sl_bytes)
        return pack(fmt,
	    self.original_network_id,
	    0xFF,
            sl_bytes,
            )

######################################################################
class service_loop_item(DVBobject):

    def pack(self):
    
        # pack service_descriptor_loop
        sdl_bytes = string.join(
            map(lambda x: x.pack(),
                self.service_descriptor_loop),
            "")

        fmt = "!HBH%ds" % len(sdl_bytes)
        return pack(fmt,
                    self.service_ID,
                    0xFC | (self.EIT_schedule_flag << 1) | (self.EIT_present_following_flag),
		    (self.running_status << 13) | (self.free_CA_mode << 12) | (len(sdl_bytes) & 0x0FFF),
                    sdl_bytes,
                    )



#! /usr/bin/env python

# This file is part of the dvbobjects library.
# 
# Copyright © 2005-2013 Lorenzo Pallara l.pallara@avalpa.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

import string
from dvbobjects.MPEG.Section import Section
from dvbobjects.utils.MJD import *
from dvbobjects.DVB.Descriptors import *

EIT_ACTUAL_TS_PRESENT_FOLLOWING = 0x4E
EIT_ANOTHER_TS_PRESENT_FOLLOWING = 0x4F
EIT_ACTUAL_TS = 0x50 #to 0x5F
EIT_ANOTHER_TS = 0x60 #to 0x6F
EIT_ACTUAL_TS_SCHEDULE14 = 0x50
EIT_ACTUAL_TS_SCHEDULE58 = 0x51

######################################################################
class event_information_section(Section):
    
    section_max_size = 4096

    def pack_section_body(self):

        self.table_id_extension = self.service_id
	self.last_table_id = self.table_id
	
        # pack event_loop
        el_bytes = string.join(
            map(lambda x: x.pack(),
                self.event_loop),
            "")

        fmt = "!HHBB%ds" % len(el_bytes)
        return pack(fmt,
	    self.transport_stream_id,
	    self.original_network_id,
	    self.segment_last_section_number,
	    self.last_table_id,
            el_bytes,
            )

######################################################################
class event_loop_item(DVBobject):

    def pack(self):
    
        # pack event_descriptor_loop
        edl_bytes = string.join(
            map(lambda x: x.pack(),
                self.event_descriptor_loop),
            "")

	# convert to MJD
	date = MJD_convert(self.start_year, self.start_month, self.start_day)

        fmt = "!HHBBBBBBH%ds" % len(edl_bytes)
        return pack(fmt,
                    self.event_id,
		    date,
		    self.start_hours,
		    self.start_minutes,
		    self.start_seconds,
		    self.duration_hours,
		    self.duration_minutes,
		    self.duration_seconds,
		    (self.running_status << 13) | (self.free_CA_mode << 12) | (len(edl_bytes) & 0x0FFF),
                    edl_bytes,                    
		    )



#! /usr/bin/env python

# This file is part of the dvbobjects library.
# 
# Copyright  2000-2001, GMD, Sankt Augustin
# -- German National Research Center for Information Technology 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

import string
from dvbobjects.MPEG.Section import Section
from dvbobjects.utils import *

######################################################################
class application_information_section(Section):

    table_id = 0x74
    
    section_max_size = 1024
    

    def pack_section_body(self):
    
        self.private_indicator = 1

        # pack common_descriptor_loop
        cdl_bytes = string.join(
            map(lambda x: x.pack(),
                self.common_descriptor_loop),
            "")

        # pack applicaton_loop
        apl_bytes = string.join(
            map(lambda x: x.pack(),
                self.application_loop),
            "")

        self.table_id_extension = self.application_type # ???

        fmt = "!H%dsH%ds" % (len(cdl_bytes), len(apl_bytes))
        return pack(fmt,
            0xF000 | (len(cdl_bytes) & 0x0FFF),
            cdl_bytes,
            0xF000 | (len(apl_bytes) & 0x0FFF),
            apl_bytes,
            )

######################################################################
class application_loop_item(DVBobject):

    def pack(self):
    
        # pack application_descriptors_loop
        adl_bytes = string.join(
            map(lambda x: x.pack(),
                self.application_descriptors_loop),
            "")

        fmt = "!LHBH%ds" % len(adl_bytes)
        return pack(fmt,
                    self.organisation_id,
                    self.application_id,
                    self.application_control_code,
                    0xF000 | (len(adl_bytes) & 0x0FFF),
                    adl_bytes,
                    )



#! /usr/bin/env python

# This file is part of the dvbobjects library.
# 
# Copyright 2010-2013 Lorenzo Pallara l.pallara@avalpa.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

import string
from dvbobjects.MPEG.Section import Section
from dvbobjects.utils import *

######################################################################
class master_guide_section(Section):

    table_id = 0xC7
    
    section_max_size = 4096

    def pack_section_body(self):
    
        # pack tables_loop
        tl_bytes = string.join(
            map(lambda x: x.pack(),
                self.tables_loop),
            "")

        # pack descriptors_loop
        dl_bytes = string.join(
            map(lambda x: x.pack(),
                self.descriptors_loop),
            "")

        self.table_id_extension = 0
        self.private_indicator = 1

        fmt = "!BH%dsH%ds" % (len(tl_bytes), len(dl_bytes))
        return pack(fmt,
    	    self.ATSC_protocol_version,
            len(self.tables_loop),
            tl_bytes,
            0xF000 | (len(dl_bytes) & 0x0FFF),
            dl_bytes,
            )

######################################################################
class table_loop_item(DVBobject):

    def pack(self):
    
        # pack transport_descriptor_loop
        dl_bytes = string.join(
            map(lambda x: x.pack(),
                self.descriptors_loop),
            "")

        fmt = "!HHBLH%ds" % len(dl_bytes)
        return pack(fmt,
    		    self.table_type,
    		    0xE000 | (self.table_type_pid & 0x1FFF),
    		    0xE0 | (self.table_type_version_number & 0x1F),
    		    self.number_bytes,
                    0xF000 | (len(dl_bytes) & 0x0FFF),
                    dl_bytes,
                    )



#! /usr/bin/env python

# This file is part of the dvbobjects library.
# 
# Copyright 2000-2001, GMD, Sankt Augustin
# -- German National Research Center for Information Technology 
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

from dvbobjects.utils import *
import Tap
import string

######################################################################
class ModuleInfo(DVBobject):

    ModuleTimeOut = 0xFFFFFFFF
    BlockTimeOut  = 0xFFFFFFFF
#   MinBlockTime  = 0x000061a8
    MinBlockTime  = 0x00000001

    userInfo = ""

    def __init__(self, **kwargs):
        # Initialize SuperClass
        apply(DVBobject.__init__, (self,), kwargs)
        self.taps = [
            Tap.object_tap(
                assocTag = self.assocTag,
                )
            ]

    def pack(self):

        assert len(self.taps) >= 1      # MHP
        taps_bytes = string.join(map(lambda t: t.pack(),
                                     self.taps),
                                 "")
        user_info_bytes = string.join(map(lambda ui: ui.pack(),
                                     self.userInfo),
                                 "")

        FMT =("!"
              "L"                       # ModuleTimeOut
              "L"                       # BlockTimeOut
              "L"                       # MinBlockTime
              "B"                       # taps_count
              "%ds"                     # taps
              "B"
	      "%ds"                    # userInfo
              ) % (
            len(taps_bytes),
            len(user_info_bytes),
            )

        return pack(FMT,
                    self.ModuleTimeOut,
                    self.BlockTimeOut,
                    self.MinBlockTime,
                    len(self.taps),
                        taps_bytes,
                        len(user_info_bytes),
                    user_info_bytes,
                    )




#! /usr/bin/env python

# This file is part of the dvbobjects library.
# 
# Copyright 2013 Lorenzo Pallara l.pallara@avalpa.com
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA

import string
from dvbobjects.MPEG.Section import Section
from dvbobjects.ATSC.Loops import *
from dvbobjects.utils.MJD import *

######################################################################
class extended_text_table_section(Section):

    table_id = 0xCC

    section_max_size = 4093

    def pack_section_body(self):
	self.table_id_extension = self.ETT_table_id_extension
	self.protocol_version = 0
	
	extended_text_message_bytes = self.extended_text_message.pack()
	
	fmt = "!BL%ds" % len(extended_text_message_bytes)
	return pack(fmt,
	    self.protocol_version,
	    self.ETM_id,
	    extended_text_message_bytes,
            )

